/*
  Copyright 2003 Rikard Bjrklind, Mikael Gransell

  This file is part of dc-qt.

  dc-qt is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  dc-qt is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with dc-qt; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "connect_dlg.h"
#include "dc_parse.h"
#include "hub_lv_item.h"
#include "dc_settings.h"
#include "util.h"

#include <qlistview.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qtabwidget.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qlistbox.h>
#include <qinputdialog.h>
#include <qpopupmenu.h>
#include <qlineedit.h>
#include <qlineedit.h>
#include <qlabel.h>

using namespace std;

ConnectDlg::ConnectDlg(dc_hub *h, 
	QWidget *p /*= 0*/, const char *name /*= 0*/, bool modal /*= false*/)
	: QDialog(p, name, modal)
{	
    setCaption("dc-qt: Connect");
	// this object contains the listing of hubs
	hubs = h;

	connect(hubs,SIGNAL(got_list()),this,SLOT(update_hublist()));
	
	tab = new QTabWidget(this,"connection tab");
	
	QVBox *tab1 = new QVBox;
	QHBox *tab2 = new QHBox;
	QHBox *tab3 = new QHBox;

	// Create tab 1.
	
	// list view of hubs in the first row
	// three buttons side by side along the bottom
	gl_layout = new QGridLayout(this, 3, 3);
	
	lv_hubs = new QListView(tab1, "connect_dlg_hubs");
	lv_hubs->addColumn("Hub Name");
	lv_hubs->addColumn("Hub IP");
	lv_hubs->addColumn("Comment");
	lv_hubs->addColumn("Users");

	lv_hubs->setColumnWidthMode(1, QListView::Manual);
	lv_hubs->setColumnWidthMode(2, QListView::Manual);
	lv_hubs->setColumnWidthMode(3, QListView::Manual);

	lv_hubs->setColumnWidth(1, 150);
	lv_hubs->setColumnWidth(2, 300);
	lv_hubs->setColumnWidth(3, 50);

	// load the hub lists
//	hubs->load_list_from_disk();
	hubs->load_running_hubs();
	refresh_listing();

	// add the public hubs to the listview
	public_hubs = hubs->get_hubs();
	add_hubs_to_list(public_lv_items, lv_hubs, public_hubs);

	menu = new QPopupMenu(lv_hubs,"connection menu");
	menu->insertItem("Connect",0);
	menu->insertItem("Add to favourites",1);
	connect(menu,SIGNAL(activated(int)),SLOT(menu_clicked(int)));
	connect(lv_hubs,SIGNAL(rightButtonClicked(QListViewItem*,const QPoint&,int)),
		SLOT(show_menu(QListViewItem*,const QPoint&,int)));
	connect(lv_hubs,SIGNAL(doubleClicked(QListViewItem*)),this,SLOT(accept()));

	tab->addTab(tab1,"&Public");

	// Create tab 2... this is one mess, need to shape up the layout later.
	QVBox *vbox = new QVBox(tab2);
	QPushButton *newBtn = new QPushButton("New",vbox);
	QPushButton *editBtn = new QPushButton("Edit",vbox);
	QPushButton *removeBtn = new QPushButton("Remove",vbox);
	vbox->setMaximumWidth(100);
	favList = new QListView(tab2,"favourites list");
	favList->addColumn("Name");	
	favList->addColumn("Address");
	vbox->layout()->addItem(new QSpacerItem(10,300));
	favList->setMaximumWidth(400);
	connect(favList,SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(accept()));

	for(int i=0;i < settings.num_favourites();i++) 
	{
		QString name,address,p;
		settings.get_favourite(i,name,address,p);
		//favList->insertItem(name);
		QListViewItem *item = new QListViewItem(favList,name,address);
	}
	
	tab->addTab(tab2,"&Favourites");
 	tab2->setStretchFactor(favList,20);
	tab2->setStretchFactor(vbox,1);
	QSpacerItem *spacer = new QSpacerItem(300,10,QSizePolicy::Expanding,QSizePolicy::Expanding);
 	spacer->setAlignment(Qt::AlignRight);
 	tab2->layout()->addItem(spacer);

	connect(newBtn, SIGNAL(clicked()), this, SLOT(addFavourite()));
	connect(editBtn, SIGNAL(clicked()), this, SLOT(editFavourite()));
	connect(removeBtn, SIGNAL(clicked()), this, SLOT(removeFavourite()));
	
	// Tab 3: Running hubs
	QVBox *running_vbox = new QVBox(tab3);
	run_lv = new QListView(tab3, "running list");
	run_lv->addColumn("Hub Name");
	run_lv->addColumn("Hub IP");
	run_lv->addColumn("Comment");
	run_lv->addColumn("Users");
	running_vbox->layout()->addItem(new QSpacerItem(10, 300));
	
	// add the running hubs to the listview
	running_hubs = hubs->get_running_hubs();
	add_hubs_to_list(running_lv_items, run_lv, running_hubs);
	
	tab->addTab(tab3, "Running");
	tab3->layout()->setSpacing(0);

	gl_layout->addMultiCellWidget(tab, 0, 0, 0, 2);
	
	// Add the rest - non-tab stuff
	pb_ok = new QPushButton("C&onnect", this);
	pb_cancel = new QPushButton("&Cancel", this);
	pb_get_hubs = new QPushButton("&Refresh Listing", this);

	quickConnectEdit = new QLineEdit(this,"quickconnectedit");

	gl_layout->addWidget( new QLabel("Quick Connect:",this), 1,0);
	gl_layout->addWidget( quickConnectEdit ,1, 1);
	gl_layout->addWidget(pb_ok, 2, 1);
	gl_layout->addWidget(pb_cancel, 2, 2);
	gl_layout->addWidget(pb_get_hubs, 2, 0);

	// size dialog
	setMaximumWidth(1280);

	connect(quickConnectEdit,SIGNAL(returnPressed()),this,SLOT(accept()));	
	connect(pb_ok, SIGNAL(clicked()), SLOT(accept()));
	connect(pb_cancel, SIGNAL(clicked()), SLOT(reject()));
	connect(pb_get_hubs, SIGNAL(clicked()), SLOT(refresh_listing()));
}

ConnectDlg::~ConnectDlg()
{
	clear_list(public_lv_items);
	clear_list(running_lv_items);
}

// given a list of hub_info*, add each hub to the given listview
void ConnectDlg::add_hubs_to_list(vector<hub_lv_item*> &l, QListView *lv, const vector<hub_info*> &h)
{
	for (int i = 0; i < h.size(); i++)
		l.push_back(new hub_lv_item(lv, h[i]->name, h[i]->ip, h[i]->comment, h[i]->users));
}

// clear the list of hubs, and delete each object, which removes
// the items from the listview
void ConnectDlg::clear_list(vector<hub_lv_item*> &l)
{
	while (l.size() > 0)
	{
		hub_lv_item* h = l.back();
		l.pop_back();
		delete h;
	}
}

// return the currently selected hub
// if no hub is selected, return an empty hub_info object
const hub_info ConnectDlg::selected_hub()
{
     // Check if there's anything in the quick connect field
     if(!quickConnectEdit->text().isEmpty())
     {
	  return hub_info("",quickConnectEdit->text(),"","");
     }

     // we're on the public hub tab
     if(tab->currentPageIndex() == 0) 
     {
	  if (!lv_hubs->selectedItem())
	       return hub_info("", "", "", "");
	  
	  return hubs->get_public_hub(lv_hubs->selectedItem()->text(HUBIP));
     }
     // we're on the favourites hub tab
     if(tab->currentPageIndex() == 1) 
     {
	  if(favList->currentItem() == NULL) 
	       return hub_info("", "", "", "");
	  return settings.get_favourite(favList->selectedItem()->text(0));
     }
     // we're on the running hub tab
//	if(tab->currentPageIndex() == 2)
     {
	  if (!run_lv->currentItem())
	       return hub_info("", "", "", "");
	  return hubs->get_running_hub(run_lv->selectedItem()->text(HUBIP));
     }
}

// return true iff the running hub tab is selected
bool ConnectDlg::selected_running()
{
	return tab->currentPageIndex() == 2;
}

// reload the hub list from the web
void ConnectDlg::refresh_listing()
{
	// clear the current hubs
	clear_list(public_lv_items);
	// do the actual getting of the list
	hubs->load_list_from_web();
}

void ConnectDlg::update_hublist()
{
  // add the hubs to the listview
  vector<hub_info*> h = hubs->get_hubs();
  add_hubs_to_list(public_lv_items, lv_hubs, h);
}

void ConnectDlg::addFavourite()
{	
     bool ok;
     QString name = QInputDialog::getText("Enter an alias","Enter an alias:",QLineEdit::Normal,QString::null,&ok);
     if(!ok) return;
     QString address = QInputDialog::getText("Enter address","Enter a hostname or IP:",QLineEdit::Normal,QString::null,&ok);
     if(!ok) return;
     QString password = QInputDialog::getText("Enter password","Enter a password or leave blank for none:",QLineEdit::Password,QString::null,&ok);
     if(!ok) return;
     QListViewItem *item = new QListViewItem(favList,name,address);
     settings.add_favourite(name,address,password);
}

void ConnectDlg::editFavourite()
{
	QString current_name = favList->selectedItem()->text(0);
	QString current_ip = settings.get_favourite(current_name).ip;
	QString current_pwd = settings.get_favourite(current_name).pwd;

	bool ok;
	QString name = QInputDialog::getText("Enter an alias","Enter an alias:",QLineEdit::Normal,
						current_name,&ok);
	if(!ok) return;
	QString address = QInputDialog::getText("Enter address","Enter a hostname or IP:",
						QLineEdit::Normal,current_ip,&ok);
	if(!ok) return;

	QString pwd = QInputDialog::getText("Enter password","Enter password or leave blank for none:",
					    QLineEdit::Password,current_pwd,&ok);
	if(!ok) return;

	settings.add_favourite(name,address,pwd);

	favList->findItem(current_name,0)->setText(0,name);
	favList->findItem(name,0)->setText(1,address);
}

void ConnectDlg::removeFavourite()
{
	QListViewItem *item = favList->currentItem();
	settings.del_favourite( item->text(0) );
	favList->takeItem( item );
	delete item;
}

void ConnectDlg::menu_clicked(int i)
{
    if(i==0) accept();
    else if(i==1)
	if(last_clicked)
	{
		QListViewItem *item = new QListViewItem(favList,
							last_clicked->text(0),
							last_clicked->text(1));	
		settings.add_favourite(last_clicked->text(0),
					last_clicked->text(1),
				       QString::null);
	}
}

void ConnectDlg::show_menu(QListViewItem* item,const QPoint &p, int)
{
	menu->popup(p);
	last_clicked = item;
}

