/*
  Copyright 2003 Rikard Bjrklind, Mikael Gransell

  This file is part of dc-qt.

  dc-qt is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  dc-qt is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with dc-qt; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <qlayout.h>
#include <qhbox.h>
#include <qpushbutton.h>
#include <qfileinfo.h>
#include <qmessagebox.h>
#include <qsizepolicy.h> 
#include <list>
#include "searchdlg.h"
#include "debugdlg.h"
#include "filetree.h"
#include "searchwdgt.h"
#include "util.h"
#include "filelistitem.h"
#include "dc_settings.h"
#include "FileExistsDlg.h"

using namespace std;

SearchDlg::SearchDlg( QWidget* parent, const char* name ) : QDialog( parent, name )
{
//    setWFlags( WType_TopLevel /*WStyle_Customize | WStyle_MinMax*/ );
     QGridLayout* layout = new QGridLayout( this, 2, 1 );

     tabs = new QTabWidget( this, "Search tabs" );

     // Create the listviews
     fileList = new QListView( tabs, "Search List" );
     fileList->addColumn( "User" );
//     fileList->addColumn( "Name",200 );
     fileList->addColumn( "Name" );
     fileList->addColumn( "Size" );
     fileList->addColumn( "Slots" );
     fileList->addColumn( "Path" );
     fileList->setAllColumnsShowFocus(true);
     //  list->setResizeMode(QListView::LastColumn);
     //   fileList->setHScrollBarMode(QScrollView::AlwaysOff);
     //fileList->setColumnWidthMode(0,QListView::Manual);
     //fileList->setColumnWidthMode(1,QListView::Manual);
     //fileList->setColumnWidthMode(2,QListView::Manual);
     //fileList->setColumnWidthMode(3,QListView::Manual);

     tree = new FileTree( tabs, "Search Tree" );
    
     // Add the lists to our tabwidget
     tabs->addTab( fileList, "List view" );
     tabs->addTab( tree, "Tree view" );
     
     layout->addWidget( tabs, 1, 0 );
     
     // Create the search widget
     searchWdgt = new SearchWdgt( this, "Search Widget" );
     searchWdgt->setSizePolicy( QSizePolicy( QSizePolicy::Maximum, QSizePolicy::Minimum) );

     QObject::connect(searchWdgt,SIGNAL(newSearch(const QString&, bool)),this,SLOT(newSearch(const QString&,bool)));
     layout->addWidget( searchWdgt, 0, 0 );

     connect(fileList, SIGNAL(doubleClicked(QListViewItem*)),
	     this, SLOT(downloadFile(QListViewItem*)));
     connect(fileList, SIGNAL(contextMenuRequested ( QListViewItem*, const QPoint& , int  )), 
	     this, SLOT(dispMenu(QListViewItem* , const QPoint& , int )));

     connect(tree, SIGNAL(downloadFile( const QString&, const QString&, const QString&, unsigned long int)),
	     this, SIGNAL(downloadFile( const QString&, const QString&, const QString&, unsigned long int)));
     connect(tree, SIGNAL(getUserFileList( const QString& )),
	     this, SIGNAL(getUserFiles( const QString& )));

     menu = new QPopupMenu(this, "user_lv_menu");
     menu->insertItem( "Download File", this, SLOT(downloadFile()),0,0,0 );
     menu->insertItem( "Download Directory", this, SLOT(downloadDirectory()),0,1,1 );
     menu->insertSeparator(2);
     menu->insertItem( "Get File List", this, SLOT(getUserFiles()),0,2,3 );
     menu->setItemEnabled(1,false);
     resize(400,400);
} 

void SearchDlg::newSearch(const QString& s,bool multiHub)
{
    multiSearch = multiHub;
     fileList->clear();
     tree->clear();
     emit search( s, multiHub );
}

void SearchDlg::addSearchResult( const QString& user, const QString& file, 
				 const QString& size, const QString& slot )
{
    // Split file into path / name
    QString fileName = file.section('\\',-1,-1);
    QString path = file.section('\\',0,-2);
    fileList->insertItem( new FileListItem( fileList, user, fileName, size, slot, path ) );

    tree->addItemToTree( user, file, size, slot );
}

/// Called when downloading from the listview
void SearchDlg::downloadFile( QListViewItem* item )
{
     FileListItem *i = (FileListItem*)item;
     QString localfile = checkFile( i->text(4) + "\\" + i->text(1),i->Size() );
     if( localfile!=QString::null )
     {
	  QString user = i->text( 0 );
	  emit downloadFile( user, i->text(4) + "\\" + i->text(1), localfile , i->Size());
     }
}

/// Called when downloading a directory from the listview
void SearchDlg::downloadDirectory()
{
/*    FileListItem *item = (FileListItem *)fileList->currentItem();
    if( item )
    {
	// Build a list of files in this directory
	list<FileInfo> files;
	QString nick = item->text(0);
	QString dir = item->text(4);
	
	item = (FileListItem *)fileList->firstChild();
	while(item)
	{
	    if(item->text(0) == nick && item->text(4) == dir)
	    {
		// Dir and nick matches the clicked item, get this file
		FileInfo finfo;
		finfo.user = nick;
		finfo.file = item->text(4) + "\\" + item->text(1);
		finfo.localFile = checkFile( finfo.file, item->Size() );
		finfo.size = item->Size();
		files.push_back( finfo );
//		emit downloadFile( nick, file, localfile, item->Size()); 
	    }
	    item = (FileListItem *)item->itemBelow();
	}
	emit downloadFiles( files );
	}*/
}

void SearchDlg::downloadFile()
{
     FileListItem *item = (FileListItem *)fileList->currentItem();
     if( item )
     {
	 QString localfile = checkFile( item->text(4) + "\\" + item->text(1),item->Size() );
	 if( localfile!=QString::null )
	 {
	     QString user = item->text( 0 );
	     QString file = item->text(4) + "\\" + item->text( 1 );
	     QString size = item->text( 2 );
	     emit downloadFile( user, file, localfile, item->Size() );
	 }
     }
     
}

void SearchDlg::dispMenu( QListViewItem* item, const QPoint& p, int column )
{
     if( !item ) // We didn't click an item but somewhere else in the list
	  return;
     // Popup the menu
     menu->popup( p );
}

void SearchDlg::getUserFiles()
{
     QListViewItem* item = fileList->currentItem();
     if( item )
     {
	  QString user = item->text( 0 );
	  emit getUserFiles( user );
     }

}

/*
 *
 * $Log: searchdlg.cc,v $
 * Revision 1.6  2003/08/22 16:45:20  olof
 * hola
 *
 * Revision 1.5  2003/08/18 12:19:43  estrato
 * Old transfer dialog now completely replaced, but some functionality is gone. Will add complete functionality in next commit.
 *
 * Revision 1.4  2003/07/29 17:47:00  olof
 * stuff
 *
 * Revision 1.3  2003/06/26 12:26:00  olof
 * multi source downloads
 *
 * Revision 1.2  2003/06/24 17:00:05  olof
 * multischeiss
 *
 * Revision 1.1.1.1  2003/06/24 10:16:17  olof
 * remake of cvs tree
 *
 * Revision 1.11  2003/06/16 19:48:25  olof
 * multihub search and password
 *
 * Revision 1.10  2003/06/16 12:18:12  olof
 * loads
 *
 * Revision 1.9  2003/06/11 22:14:21  olof
 * transfers ok again
 *
 * Revision 1.8  2003/06/06 11:34:11  olof
 * mu
 *
 * Revision 1.7  2003/05/08 12:34:14  olof
 * GPL header added
 *
 * Revision 1.6  2003/05/07 10:17:00  olof
 * listview fixes
 *
 * Revision 1.5  2003/05/07 00:11:54  olof
 * massa mandelmassa
 *
 * Revision 1.4  2003/04/16 20:54:47  olof
 * user file list fixed
 *
 * Revision 1.3  2003/03/18 14:27:29  mickeg
 * Filetree working
 * Hopefully the userfilelisting also works.
 *
 * Revision 1.2  2003/03/16 21:00:49  mickeg
 * Added functionality for the tree view of the search results
 * Still some bugs
 * Doesn't iterate through the children accurately
 *
 * Revision 1.1  2003/03/04 22:10:40  olof
 * dcqt2 creation
 *
 * Revision 1.4  2003/02/23 16:54:49  ribj4077
 * Added some features
 *
 * Revision 1.3  2003/02/18 15:47:33  migr8915
 * Bastard rightclick signal not working
 * Should be able to download by doubleclicking item
 *
 *
 */
